<?php
/**
 * Client portal services controller
 *
 * @package blesta
 * @subpackage blesta.app.controllers
 * @copyright Copyright (c) 2010, Phillips Data, Inc.
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/ Blesta
 */
class ClientServices extends AppController {
	
	/**
	 * Pre-action
	 */
	public function preAction() {
		parent::preAction();
		
		// Require login
		$this->requireLogin();
		
		// Load models, language
		$this->uses(array("Clients", "Packages", "Services"));
		Language::loadLang("client_services");
		
		$this->client = $this->Clients->get($this->Session->read("blesta_client_id"));
		
		// Attempt to set the page title language
		if ($this->client) {
			try {
				$language = Language::_("ClientServices." . Loader::fromCamelCase($this->action ? $this->action : "index") . ".page_title", true, $this->client->id_code);
				$this->structure->set("page_title", $language);
			}
			catch(Exception $e) {
				// Attempting to set the page title language has failed, likely due to
				// the language definition requiring multiple parameters.
				// Fallback to index. Assume the specific page will set its own page title otherwise.
				$this->structure->set("page_title", Language::_("ClientServices.index.page_title", true), $this->client->id_code);
			}
		}
		else
			$this->redirect($this->base_uri);
	}
	
	/**
	 * List services
	 */
	public function index() {
		$status = (isset($this->get[0]) ? $this->get[0] : "active");
		$page = (isset($this->get[1]) ? (int)$this->get[1] : 1);
		$sort = (isset($this->get['sort']) ? $this->get['sort'] : "date_added");
		$order = (isset($this->get['order']) ? $this->get['order'] : "desc");
		
		$services = $this->Services->getList($this->client->id, $status, $page, array($sort => $order), false);
		$total_results = $this->Services->getListCount($this->client->id, $status, false);
		
		// Set the number of services of each type, not including children
		$status_count = array(
			'active' => $this->Services->getStatusCount($this->client->id, "active", false),
			'canceled' => $this->Services->getStatusCount($this->client->id, "canceled", false),
			'pending' => $this->Services->getStatusCount($this->client->id, "pending", false),
			'suspended' => $this->Services->getStatusCount($this->client->id, "suspended", false),
		);
		
		// Set language for periods
		$periods = $this->Packages->getPricingPeriods();
		foreach ($this->Packages->getPricingPeriods(true) as $period=>$lang)
			$periods[$period . "_plural"] = $lang;
		
		$this->set("periods", $periods);
		$this->set("client", $this->client);
		$this->set("status", $status);
		$this->set("services", $services);
		$this->set("status_count", $status_count);
		$this->set("widget_state", isset($this->widgets_state['services']) ? $this->widgets_state['services'] : null);
		$this->set("sort", $sort);
		$this->set("order", $order);
		$this->set("negate_order", ($order == "asc" ? "desc" : "asc"));
		
		// Overwrite default pagination settings
		$settings = array_merge(Configure::get("Blesta.pagination_client"), array(
				'total_results' => $total_results,
				'uri'=>$this->Html->safe($this->base_uri . "services/index/" . $status . "/[p]/"),
				'params'=>array('sort'=>$sort,'order'=>$order)
			)
		);
		$this->helpers(array("Pagination"=>array($this->get, $settings)));
		$this->Pagination->setSettings(Configure::get("Blesta.pagination_ajax"));
		
		if ($this->isAjax())
			return $this->renderAjaxWidgetIfAsync(isset($this->get['whole_widget']) ? null : (isset($this->get[1]) || isset($this->get['sort'])));
	}
	
	/**
	 * Manage service
	 */
	public function manage() {
		
		$this->uses(array("ModuleManager"));
		
		// Ensure we have a service
		if (!($service = $this->Services->get((int)$this->get[0])) || $service->client_id != $this->client->id)
			$this->redirect($this->base_uri);
		
		$package = $this->Packages->get($service->package->id);
		$module = $this->ModuleManager->initModule($service->package->module_id);
		$module->base_uri = $this->base_uri;
		
		$method = isset($this->get[1]) ? $this->get[1] : null;
		
		// Set sidebar tabs
		$this->buildTabs($service, $package, $module, $method);
		
		// Get tabs
		$client_tabs = $module->getClientTabs($package);
		
		$tab_view = null;
		// Load/process the tab request
		if ($method && key_exists($method, $client_tabs) && is_callable(array($module, $method))) {
			// Set the module row used for this service
			$module->setModuleRow($module->getModuleRow($service->module_row_id));
			
		 	$tab_view = $module->{$method}($package, $service, $this->get, $this->post, $this->files);
			
			if (($errors = $module->errors()))
				$this->setMessage("error", $errors);
			elseif (!empty($this->post))
				$this->setMessage("success", Language::_("ClientServices.!success.manage.tab_updated", true));
		}
		
		$this->set("tab_view", $tab_view);
		
		// Set language for periods
		$periods = $this->Packages->getPricingPeriods();
		foreach ($this->Packages->getPricingPeriods(true) as $period=>$lang)
			$periods[$period . "_plural"] = $lang;
		
		// Set whether the client can cancel a service
		// First check whether the service is already canceled
		if (!$service->date_canceled || ($service->date_canceled && strtotime($this->Date->cast($service->date_canceled, "date_time")) > strtotime(date("c")))) {
			// Service has not already been canceled, check whether the setting is enabled for clients to cancel services
			$client_cancel_service = $this->client->settings['clients_cancel_services'] == "true";
		}
		else {
			// Service is already canceled, can't cancel it again
			$client_cancel_service = false;
		}
		
		// Set partial for the service information box
		$service_params = array(
			'periods' => $periods,
			'service' => $service,
			'next_invoice_date' => $this->Services->getNextInvoiceDate($service->id),
			'client_cancel_service' => $client_cancel_service
		);
		$this->set("service_infobox", $this->partial("client_services_service_infobox", $service_params));
		
		$this->set("service", $service);
		$this->set("package", $package);
		
		// Display a notice regarding the service being suspended/canceled
		$error_notice = array();
		if (!empty($service->date_suspended))
			$error_notice[] = Language::_("ClientServices.manage.text_date_suspended", true, $this->Date->cast($service->date_suspended));
		if (!empty($service->date_canceled)) {
			$scheduled = false;
			if ($this->Date->toTime($this->Date->cast($service->date_canceled)) > $this->Date->toTime($this->Date->cast(date("c"))))
				$scheduled = true;
			$error_notice[] = Language::_("ClientServices.manage.text_date_" . ($scheduled ? "to_cancel" : "canceled"), true, $this->Date->cast($service->date_canceled));
		}
		if (!empty($error_notice))
			$this->setMessage("error", array('notice' => $error_notice));
		
		if ($this->isAjax())
			return $this->renderAjaxWidgetIfAsync(isset($this->get['whole_widget']) ? null : false);
	}
	
	/**
	 * Cancel Service
	 */
	public function cancel() {
		$this->uses(array("Currencies", "Users"));
		
		$client_can_cancel_service = isset($this->client->settings['clients_cancel_services']) && $this->client->settings['clients_cancel_services'] == "true";
		
		// Ensure we have a service
		if (!$client_can_cancel_service || !($service = $this->Services->get((int)$this->get[0])) || $service->client_id != $this->client->id) {
			if ($this->isAjax())
				exit();
			$this->redirect($this->base_uri);
		}
		
		if (!empty($this->post)) {
			$data = $this->post;
			
			// Verify that client's password is correct, set $errors otherwise
			if ($this->Users->auth($this->Session->read("blesta_id"), array('password' => $this->post['password']))) {
				// Cancel the service
				switch ($this->post['cancel']) {
					case "now":
						$this->Services->cancel($service->id, $data);
						break;
					case "term":
						// Cancel at end of service term
						$data['date_canceled'] = "end_of_term";
						$this->Services->cancel($service->id, $data);
						break;
					default:
						// Do not cancel
						$this->Services->unCancel($service->id);
						break;
				}
			}
			else {
				$errors = array('password' => array('mismatch' => Language::_("ClientServices.!error.password_mismatch", true)));
			}
			
			if (!empty($errors) || ($errors = $this->Services->errors())) {
				$this->flashMessage("error", $errors);
			}
			else {
				// Success
				$this->flashMessage("message", Language::_("ClientServices.!success.service_" . ($this->post['cancel'] == "term" ? "schedule_" : ($this->post['cancel'] == "" ? "not_" : "")) . "canceled", true));
			}
			
			$this->redirect($this->base_uri . "services/manage/" . $service->id . "/");
		}
		
		// Set language for periods
		$periods = $this->Packages->getPricingPeriods();
		foreach ($this->Packages->getPricingPeriods(true) as $period=>$lang)
			$periods[$period . "_plural"] = $lang;
		
		// Set whether the client can cancel a service
		// First check whether the service is already canceled
		if (!$service->date_canceled || ($service->date_canceled && strtotime($this->Date->cast($service->date_canceled, "date_time")) > strtotime(date("c")))) {
			// Service has not already been canceled, check whether the setting is enabled for clients to cancel services
			$client_cancel_service = $this->client->settings['clients_cancel_services'] == "true";
		}
		else {
			// Service is already canceled, can't cancel it again
			$client_cancel_service = false;
		}
		
		// Set partial for the service information box
		$service_params = array(
			'periods' => $periods,
			'service' => $service,
			'next_invoice_date' => $this->Services->getNextInvoiceDate($service->id),
			'client_cancel_service' => $client_cancel_service
		);
		$this->set("service_infobox", $this->partial("client_services_service_infobox", $service_params));
		
		// Set the cancellation to be at the end of the term
		if (!isset($vars))
			$vars = (object)array('cancel' => "");
		
		// Set the confirmation message for canceling the service
		$cancel_messages = array(
			'now' => Language::_("ClientServices.cancel.confirm_cancel_now", true),
			'term' => Language::_("ClientServices.cancel.confirm_cancel", true)
		);
		if (isset($service->package_pricing->cancel_fee) && $service->package_pricing->cancel_fee > 0) {
			// Get the client settings
			$client_settings = $this->SettingsCollection->fetchClientSettings($service->client_id);
			
			// Get the pricing info
			if ($client_settings['default_currency'] != $service->package_pricing->currency)
				$pricing_info = $this->Services->getPricingInfo($service->id, $client_settings['default_currency']);
			else
				$pricing_info = $this->Services->getPricingInfo($service->id);
			
			// Set the formatted cancellation fee and confirmation message
			if ($pricing_info) {
				$cancellation_fee = $this->Currencies->toCurrency($pricing_info->cancel_fee, $pricing_info->currency, $this->company_id);
				
				$cancel_messages['now'] = Language::_("ClientServices.cancel.confirm_cancel_now", true) . " " . Language::_("ClientServices.cancel.confirm_cancel_now_fee", true, $cancellation_fee);
				
				if ($pricing_info->tax)
					$cancel_messages['now'] = Language::_("ClientServices.cancel.confirm_cancel_now", true) . " " . Language::_("ClientServices.cancel.confirm_cancel_now_fee_tax", true, $cancellation_fee);
			}
		}
		
		foreach ($cancel_messages as $key => $message) {
			$cancel_messages[$key] = $this->setMessage("notice", $message, true);
		}

		$this->set("service", $service);
		$this->set("package", $this->Packages->get($service->package->id));
		$this->set("vars", $vars);		
		$this->set("confirm_cancel_messages", $cancel_messages);
		
		echo $this->view->fetch("client_services_cancel");
		return false;
	}
	
	/**
	 * List Addons
	 */
	public function addons() {
		// Determine whether a service is given and may have addons
		if (!isset($this->get[0]) || !($service = $this->Services->get((int)$this->get[0])) || $service->client_id != $this->client->id)
			$this->redirect($this->base_uri);
		
		// Fetch addons
		$addon_services = $this->Services->getAllChildren($service->id);
		$available_addons = $this->getAddonPackages($service->package_group_id);
		
		// Must have addons available to view this page
		if (empty($addon_services) && empty($available_addons))
			$this->redirect($this->base_uri . "services/manage/" . $service->id . "/");
		
		// Fetch the package and module
		$this->uses(array("ModuleManager"));
		$package = $this->Packages->get($service->package->id);
		$module = $this->ModuleManager->initModule($service->package->module_id);
		$module->base_uri = $this->base_uri;
		
		// Set sidebar tabs
		$this->buildTabs($service, $package, $module, "addons");
		
		// Set language for periods
		$periods = $this->Packages->getPricingPeriods();
		foreach ($this->Packages->getPricingPeriods(true) as $period=>$lang)
			$periods[$period . "_plural"] = $lang;
		$this->set("periods", $periods);
		$this->set("statuses", $this->Services->getStatusTypes());
		$this->set("services", $addon_services);
		$this->set("service", $service);
		$this->set("package", $this->Packages->get($service->package->id));
		$this->set("client_can_create_addons", (!empty($available_addons) && (isset($this->client->settings['client_create_addons']) && $this->client->settings['client_create_addons'] == "true")));
	}
	
	/**
	 * Create an addon
	 */
	public function addAddon() {
		// Ensure a valid service was given
		$client_can_create_addon = (isset($this->client->settings['client_create_addons']) && $this->client->settings['client_create_addons'] == "true");
		if (!$client_can_create_addon || !isset($this->get[0]) || !($service = $this->Services->get((int)$this->get[0])) || $service->client_id != $this->client->id ||
			!($available_addons = $this->getAddonPackages($service->package_group_id)))
			$this->redirect($this->base_uri);
		
		// Fetch the package and module
		$this->uses(array("Invoices", "ModuleManager"));
		$package = $this->Packages->get($service->package->id);
		$module = $this->ModuleManager->initModule($service->package->module_id);
		$module->base_uri = $this->base_uri;
		
		$package_group_id = (isset($this->get['package_group_id']) ? $this->get['package_group_id'] : "");
		$addon_pricing_id = (isset($this->get['pricing_id']) ? $this->get['pricing_id'] : "");
		
		// Detect module refresh fields
		$refresh_fields = isset($this->post['refresh_fields']) && $this->post['refresh_fields'] == "true";
		
		// Go to configure service options
		if (!empty($package_group_id) && !empty($addon_pricing_id)) {
			// Determine whether the addon is valid
			$fields = $this->validateAddon($service->id, $package_group_id, $addon_pricing_id);
			if (!$fields['valid']) {
				$this->setMessage("error", Language::_("ClientServices.!error.addon_invalid", true));
			}
			elseif (!$refresh_fields && !empty($this->post)) {
				$data = $this->post;
				
				// Set pricing term selected
				$data['package_id'] = $fields['package']->id;
				$data['pricing_id'] = $fields['pricing']->id;
				$data['parent_service_id'] = $fields['parent_service']->id;
				$data['package_group_id'] = $fields['package_group']->id;
				$data['client_id'] = $this->client->id;
				$data['status'] = "pending";
				$data['use_module'] = "true";
				
				// Unset any fields that may adversely affect the Services::add() call
				unset($data['date_added'], $data['date_renews'], $data['date_last_renewed'], $data['date_suspended'],
					$data['date_canceled'], $data['notify_order'], $data['invoice_id'], $data['invoice_method'], $data['coupon_id']);
				
				// Attempt to add the addon
				if (isset($data['qty']))
					$data['qty'] = (int)$data['qty'];
				
				// Verify fields look correct in order to proceed
				$this->Services->validateService($fields['package'], $data);
				if (($errors = $this->Services->errors())) {
					$this->setMessage("error", $errors);
				}
				else {
					// Add addon
					$service_id = $this->Services->add($data, array('package_id' => $fields['package']->id));
					
					if (($errors = $this->Services->errors()))
						$this->setMessage("error", $errors);
					else {
						// Create the invoice
						$invoice_id = $this->Invoices->createFromServices($this->client->id, array($service_id), $fields['currency'], date("c"));
						
						// Redirect the client to pay the invoice
						if ($invoice_id) {
							$this->flashMessage("message", Language::_("ClientServices.!success.addon_service_created", true));
							$this->redirect($this->base_uri . "pay/method/" . $invoice_id . "/");
						}
					}
				}
			}
			
			// Set the configurable options partial for the selected addon
			$data = array_merge($this->post, array('addon' => $package_group_id . "_" . $addon_pricing_id));
			if (!empty($fields['pricing']) && ($addon_options = $this->getAddonOptions($fields, true, $data))) {
				$this->set("addon_options", $addon_options);
				$vars = (object)$data;
			}
			else
				$vars = (object)array('addon' => "");
		}
		
		// Set sidebar tabs
		$this->buildTabs($service, $package, $module, "addons");
		
		$this->set("module", $module->getModule());
		$this->set("package", $package);
		$this->set("service", $service);
		$this->set("addons", $this->getAddonPackageList($available_addons));
		$this->set("vars", (isset($vars) ? $vars : new stdClass()));
		
		// Load slider JS/CSS
		$this->Javascript->setFile("bootstrap-slider.js");
	}
	
	/**
	 * AJAX - Retrieves the configurable options for a given addon
	 *
	 * @param array $fields A list of addon fields (optional)
	 * @param boolean $return True to return this partial view, false to output as json (optional, default false)
	 * @param array $vars A list of input vars (optional)
	 * @return mixed False if addon fields are invalid; a partial view if $return is true; otherwise null
	 */
	public function getAddonOptions($fields = array(), $return = false, $vars = array()) {
		if (!$this->isAjax() && !$return) {
			header($this->server_protocol . " 401 Unauthorized");
			exit();
		}
		
		// Validate the addon
		if (!empty($this->get['package_group_id']) && !empty($this->get['pricing_id']) && !empty($this->get[0])) {
			$this->uses(array("PackageOptions"));
			
			// Determine whether the addon is valid
			$fields = $this->validateAddon((int)$this->get[0], (int)$this->get['package_group_id'], (int)$this->get['pricing_id']);
			if (!$fields['valid'])
				return false;
		}
		
		// Build the partial
		if (!empty($fields) && $fields['valid'] && $fields['module']) {
			// Set the module service fields and package option fields
			$vars = (object)$vars;
			$service_fields = $fields['module']->getClientAddFields($fields['package'], $vars);
			$package_options = $this->PackageOptions->getFields($fields['pricing']->package_id, $fields['pricing']->term, $fields['pricing']->period, $fields['pricing']->currency, $vars);
			
			$vars = array(
				'module' => $fields['module']->getModule(),
				'fields' => $service_fields->getFields(),
				'html' => $service_fields->getHtml(),
				'package_options' => $this->partial("client_services_package_options", array('fields' => $package_options->getFields()))
			);
			
			$partial = $this->partial("client_services_configure_addon", $vars);
			if ($return)
				return $partial;
			$this->outputAsJson($partial);
		}
		
		return false;
	}
	
	/**
	 * Validates that the given data is valid for a client
	 * @see ClientServices::addAddon(), ClientServices::getAddonOptions()
	 *
	 * @param int $service_id The ID of the parent service to which the addon is to be assigned
	 * @param int $package_group_id The ID of the package group
	 * @param int $pricing_id The ID of the addon's package pricing 
	 * @return array An array of fields including:
	 * 	- valid True if the addon is valid; false otherwise
	 * 	- parent_service An stdClass object representing the parent service of the addon (optional, only if valid is true)
	 * 	- package An stdClass object representing the addon package (optional, only if valid is true)
	 * 	- pricing An stdClass object representing the package pricing term (optional, only if valid is true)
	 * 	- package_group An stdClass object representing the addon package group (optional, only if valid is true)
	 * 	- module An stdClass object representing the module (optional, only if valid is true)
	 * 	- currency The currency code
	 */
	private function validateAddon($service_id, $package_group_id, $price_id) {
		$this->uses(array("ModuleManager", "PackageGroups"));
		
		// Ensure a valid addon was given
		if (!($parent_service = $this->Services->get((int)$service_id)) || $parent_service->client_id != $this->client->id ||
			!($available_addons = $this->getAddonPackages($parent_service->package_group_id, true)) ||
			!($package = $this->Packages->getByPricingId((int)$price_id)) || 
			!($package_group = $this->PackageGroups->get((int)$package_group_id)) || $package_group->company_id != $this->company_id) {
			return array('valid' => false);
		}
		
		// Confirm that the given package is an available addon
		$valid = false;
		$addon_groups = (isset($available_addons[$package_group->id]) ? $available_addons[$package_group->id] : array());
		$currency = $this->Clients->getSetting($this->client->id, "default_currency");
		$currency = $currency->value;
		$pricing = null;
		foreach ($addon_groups->addons as $addon) {
			if ($addon->id == $package->id) {
				$valid = true;
				foreach ($package->pricing as $pricing) {
					if ($pricing->id == (int)$price_id) {
						$currency = $pricing->currency;
						break;
					}
				}
				break;
			}
		}
		
		// Ensure a valid module exists
		$module = $this->ModuleManager->initModule($package->module_id, $this->company_id);
		if (!$module)
			$valid = false;
		
		return compact("valid", "parent_service", "package", "pricing", "package_group", "module", "currency");
	}
	
	/**
	 * Service Info
	 */
	public function serviceInfo() {
		
		$this->uses(array("ModuleManager"));
			
		// Ensure we have a service
		if (!($service = $this->Services->get((int)$this->get[0])) || $service->client_id != $this->client->id)
			$this->redirect($this->base_uri);
		
		$package = $this->Packages->get($service->package->id);
		$module = $this->ModuleManager->initModule($service->package->module_id);
		
		if ($module) {
			$module->base_uri = $this->base_uri;
			$module->setModuleRow($module->getModuleRow($service->module_row_id));
			$this->set("content", $module->getClientServiceInfo($service, $package));
		}
		
		// Set any addon services
		$this->set("services", $this->Services->getAllChildren($service->id));
		
		// Set language for periods
		$periods = $this->Packages->getPricingPeriods();
		foreach ($this->Packages->getPricingPeriods(true) as $period=>$lang)
			$periods[$period . "_plural"] = $lang;
		$this->set("periods", $periods);
		$this->set("statuses", $this->Services->getStatusTypes());

		echo $this->outputAsJson($this->view->fetch("client_services_serviceinfo"));
		return false;
	}
	
	/**
	 * Format module tabs into an array of tabs
	 *
	 * @param array $tabs An array of tab data
	 * @return array An array of module tabs
	 */
	private function formatModuleTabs($tabs, $service, $method) {
		$module_tabs = array();
		foreach ($tabs as $action => $link) {
			if (!is_array($link))
				$link = array('name' => $link);
			if (!isset($link['icon']))
				$link['icon'] = "fa fa-cog";
			if (!isset($link['href'])) {
				$link['href'] = $this->base_uri . "services/manage/" . $service->id . "/" . $action . "/";
				$link['class'] = "ajax";
			}
			
			$module_tabs[] = array(
				'name' => $link['name'],
				'attributes' => array('href' => $link['href'], 'class' => isset($link['class']) ? $link['class'] : ""),
				'current' => strtolower($action) == strtolower($method),
				'icon' => $link['icon']
			);
		}
		return $module_tabs;
	}
	
	/**
	 * Retrieves a list of name/value pairs for addon packages
	 *
	 * @param array $addon_packages A list of package groups containing addon packages available to the client
	 * @return array A list of name/value pairs
	 */
	private function getAddonPackageList(array $addons) {
		// Set language for periods
		$periods = $this->Packages->getPricingPeriods();
		foreach ($this->Packages->getPricingPeriods(true) as $period=>$lang)
			$periods[$period . "_plural"] = $lang;
		
		$addon_packages = array('' => Language::_("AppController.select.please", true));
		foreach ($addons as $package_group) {
			foreach ($package_group->addons as $addon) {
				// Add addon package name
				$addon_packages[] = array('name' => $addon->name, 'value' => "optgroup");
				
				foreach ($addon->pricing as $price) {
					// Set term
					$period_singular = (isset($periods[$price->period]) ? $periods[$price->period] : "");
					$period_plural = (isset($periods[$price->period . "_plural"]) ? $periods[$price->period . "_plural"] : "");
					
					if ($this->Html->ifSet($price->period) == "onetime")
						$term = $period_singular;
					else {
						$term = (isset($price->term) ? $price->term : "");
						$term = Language::_("ClientServices.addaddon.term", true, $term, ($term == 1 ? $period_singular : $period_plural));
					}
					
					// Set price, setup fee
					$cost = $this->CurrencyFormat->format($this->Html->ifSet($price->price), $this->Html->ifSet($price->currency), array('code' => false));
					$name = Language::_("ClientServices.addaddon.term_price", true, $term, $cost);
					if ($price->setup_fee > 0) {
						$setup_fee = $this->CurrencyFormat->format($price->setup_fee, $price->currency, array('code' => false));
						$name = Language::_("ClientServices.addaddon.term_price_setupfee", true, $term, $cost, $setup_fee);
					}
					
					// Add addon package
					$addon_packages[] = array('name' => $name, 'value' => $package_group->id . "_" . $price->id);
				}
			}
		}
		return $addon_packages;
	}
	
	/**
	 * Retrieves a list of all addon packages available to the client in the given package group
	 *
	 * @param int $parent_group_id The ID of the parent group to list packages for
	 * @return array An array of addon package groups containing an array of addon packages
	 */
	private function getAddonPackages($parent_group_id) {
		$this->uses(array("Packages", "PackageGroups"));
		
		$packages = array();
		$package_groups = $this->Packages->getAllAddonGroups($parent_group_id);
		
		foreach ($package_groups as $package_group) {
			$temp_packages = $this->Packages->getAllPackagesByGroup($package_group->id);
			$restricted_packages = $this->Clients->getRestrictedPackages($this->client->id);
			
			foreach ($temp_packages as $package) {
				// Check whether the client has access to this package
				if ($package->status == "inactive")
					continue;
				elseif ($package->status == "restricted") {
					$available = false;
					foreach ($restricted_packages as $restricted) {
						if ($restricted->package_id == $package->id) {
							$available = true;
							break;
						}
					}
					
					if (!$available)
						continue;
				}
				
				// Add the addon package to the list
				if (!isset($packages[$package_group->id])) {
					$packages[$package_group->id] = $package_group;
					$packages[$package_group->id]->addons = array();
				}
				$packages[$package_group->id]->addons[] = $package;
			}
		}
		
		return $packages;
	}
	
	/**
	 * Builds and sets the sidebar tabs for the service management views
	 *
	 * @param stdClass $service An stdClass object representing the service
	 * @param stdClass $package An stdClass object representing the package used by the service
	 * @param stdClass $module An stdClass object representing the module the package uses
	 * @param string $method The method to call on the module, if any
	 */
	private function buildTabs($service, $package, $module, $method) {
		// Service information tab
		$tabs = array(
			array(
				'name' => Language::_("ClientServices.manage.tab_service_info", true),
				'attributes' => array('href' => $this->base_uri . "services/manage/" . $service->id . "/", 'class' => "ajax"),
				'current' => empty($method),
				'icon' => "fa fa-info-circle"
			)
		);
		
		// Determine whether addons are accessible
		$has_addons = $this->Services->hasChildren($service->id);
		if (!$has_addons) {
			$available_addons = $this->getAddonPackages($service->package_group_id);
			$has_addons = !empty($available_addons);
		}
		
		if ($has_addons) {
			$tabs[] = array(
				'name' => Language::_("ClientServices.manage.tab_addons", true),
				'attributes' => array('href' => $this->base_uri . "services/addons/" . $service->id . "/", 'class' => "ajax"),
				'current' => ($method == "addons"),
				'icon' => "fa fa-plus-circle"
			);
		}
		
		// Get tabs
		$client_tabs = $module->getClientTabs($package);
		$tabs = array_merge($tabs, $this->formatModuleTabs($client_tabs, $service, $method));
		
		// Return to dashboard
		$tabs[] = array(
			'name' => Language::_("ClientServices.manage.tab_service_return", true),
			'attributes' => array('href' => $this->base_uri),
			'current' => false,
			'icon' => "fa fa-arrow-left"
		);
		
		$this->set("tabs", $this->partial("client_services_tabs", array('tabs' => $tabs)));
	}
}
?>